local hits = 0
local no_hit = 0
local debug = false

local function dynamic_critical_hit_chance(crit_value)
	if crit_value <= 0 then
		return false
	end

	if crit_value >= 1 then
		return true
	end

	local chance = crit_value
	local ref_chance = chance
	local crit_hit = false

	if hits >= 3 then
		ref_chance = math.max(ref_chance - (0.005 * hits), chance * 0.5)
	end

	if no_hit >= 3 then
		ref_chance = math.min(math.min(ref_chance + (0.005 * no_hit), chance * 1.5), 1)
	end

	if math.random() <= ref_chance then
		if debug then
			log("### CRITICAL HIT ### - CURRENT CHANCE: " .. ref_chance .. " HITS = "..hits.. " NO HITS = "..no_hit)
		end

		hits = hits + 1

		if no_hit >= 1 then
			no_hit = 0
			ref_chance = chance
		end

		crit_hit = true
	else
		if debug then
			log("### NON CRITICAL HIT ### - CURRENT CHANCE: " .. ref_chance .. " HITS = "..hits.. " NO HITS = "..no_hit)
		end

		no_hit = no_hit + 1

		if hits >= 1 then
			hits = 0
			ref_chance = chance
		end
	end

	return crit_hit
end

if BeardLib.Utils:ModExists("Freyah's Skills Overhaul (+ CatLib)") then
    function CopDamage:roll_critical_hit(attack_data, damage)
        if not self:can_be_critical(attack_data) then
            return false, damage
        end

        local critical_hits = self._char_tweak.critical_hits or {}
        local critical_value = (critical_hits.base_chance or 0) + managers.player:critical_hit_chance() * (critical_hits.player_chance_multiplier or 1)
        local bonus_crit_dmg_skill = managers.player:get_bonus_critical_damage()
        local critical_hit = dynamic_critical_hit_chance(critical_value)

        if critical_hit then
            local critical_damage_mul = tweak_data.upgrades.critical_base_damage + managers.player:upgrade_value("player", "critical_damage_card", 0) + bonus_crit_dmg_skill
            damage = damage * critical_damage_mul
        end

        return critical_hit, damage
    end
else
    function CopDamage:roll_critical_hit(attack_data, damage)
        if not self:can_be_critical(attack_data) then
            return false, damage
        end

        local critical_hits = self._char_tweak.critical_hits or {}
        local critical_value = (critical_hits.base_chance or 0) + managers.player:critical_hit_chance() * (critical_hits.player_chance_multiplier or 1)
        local critical_hit = dynamic_critical_hit_chance(critical_value)

        if critical_hit then
            local critical_damage_mul = critical_hits.damage_mul or self._char_tweak.headshot_dmg_mul

            if critical_damage_mul then
                damage = damage * critical_damage_mul
            else
                damage = self._health * 10
            end
        end

        return critical_hit, damage
    end
end